import argparse
from crtl.components.ann_vector_kb import ANNCandidateGenerator
from crtl.components.vectorizer import TransformerSpanVectorizer
from crtl.lkb.lkb import LexicalKnowledgeBase
from crtl.s3_file_utils.s3_file_utils import cached_path
from flask import Flask, request
from flask_restful import Resource, Api
from flask_cors import CORS

from crtl.lkb.api.controller import APIController, create_trendnet_api_controller

# controller = create_trendnet_api_controller()
lkb = LexicalKnowledgeBase.from_gzipped_jsonl(
    cached_path("s3://databrickstrenddata/crtl/lkb/trendnet_2.0.1.jsonl.gz")
)
vectorizer = TransformerSpanVectorizer(50, batch_size=512, device="cpu")
candidate_generator = ANNCandidateGenerator(vectorizer, lkb_getter=lambda: lkb)
candidate_generator.from_tar(
    cached_path("s3://databrickstrenddata/crtl/linkers/ann_cg_sap_bert.tar.gz")
)
controller = APIController(lkb, candidate_generator)


class GetConceptInfo(Resource):
    def get(self):
        args = request.args
        concept_id = args["cui"]
        return controller.get_concept_info(concept_id)


class GetTermInfo(Resource):
    def get(self):
        args = request.args
        term_id = args["tui"]
        return controller.get_term_info(term_id)


class SearchForConcept(Resource):
    def get(self):
        args = request.args
        search_term = args["search_term"]
        return controller.search_for_concept(search_term)


def get_app():
    app = Flask(__name__)
    CORS(app)
    api = Api(app)
    api.add_resource(GetConceptInfo, "/get_concept_info")
    api.add_resource(GetTermInfo, "/get_term_info")
    api.add_resource(SearchForConcept, "/search_for_concept")
    return app


def start_api(host="0.0.0.0", port=5000):
    app = get_app()
    app.run(host=host, port=port, debug=False, use_reloader=False)
    print("App is running.")
    return app


if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="API server")
    parser.add_argument("--host", type=str, default="0.0.0.0", help="Host address")
    parser.add_argument("--port", type=int, default=5001, help="Port number")
    args = parser.parse_args()
    start_api(host=args.host, port=args.port)
